@@ -6,12 +6,18 @@ Based on transformers/models/vit by HuggingFace
 
 Copyright 2021 Martins Bruveris
 """
-from dataclasses import dataclass, field
-from typing import Callable, Dict, Optional, Tuple, Union
+from dataclasses import dataclass
+from typing import List, Optional, Tuple, Union
 
 import tensorflow as tf
 
-from tfimm.layers import MLP, PatchEmbeddings, norm_layer_factory
+from tfimm.layers import (
+    MLP,
+    DropPath,
+    PatchEmbeddings,
+    interpolate_pos_embeddings,
+    norm_layer_factory,
+)
 from tfimm.models import ModelConfig, keras_serializable, register_model
 from tfimm.utils import (
     IMAGENET_DEFAULT_MEAN,
@@ -20,6 +26,8 @@ from tfimm.utils import (
     IMAGENET_INCEPTION_STD,
 )
 
+from .resnetv2 import ResNetV2, ResNetV2Config, ResNetV2Stem
+
 # model_registry will add each entrypoint fn to this
 __all__ = ["ViT", "ViTConfig"]
 
@@ -27,19 +35,24 @@ __all__ = ["ViT", "ViTConfig"]
 @dataclass
 class ViTConfig(ModelConfig):
     nb_classes: int = 1000
-    in_chans: int = 3
+    in_channels: int = 3
     input_size: Tuple[int, int] = (224, 224)
+    patch_layer: str = "patch_embeddings"
+    patch_nb_blocks: tuple = ()
     patch_size: int = 16
     embed_dim: int = 768
-    depth: int = 12
+    nb_blocks: int = 12
     nb_heads: int = 12
     mlp_ratio: float = 4.0
     qkv_bias: bool = True
     representation_size: Optional[int] = None
     distilled: bool = False
+    # Regularization
     drop_rate: float = 0.0
     attn_drop_rate: float = 0.0
-    norm_layer: str = "layer_norm"
+    drop_path_rate: float = 0.0
+    # Other parameters
+    norm_layer: str = "layer_norm_eps_1e-6"
     act_layer: str = "gelu"
     # Parameters for inference
     interpolate_input: bool = False
@@ -50,48 +63,62 @@ class ViTConfig(ModelConfig):
     first_conv: str = "patch_embed/proj"
     # DeiT models have two classifier heads, one for distillation
     classifier: Union[str, Tuple[str, str]] = "head"
-    transform_weights: Dict[str, Callable] = field(default_factory=dict)
 
     """
     Args:
-        num_classes: number of classes for classification head
-        in_chans: number of input channels
-        input_size: input image size
-        patch_size: Patch size; Image size must be multiple of patch size
+        nb_classes: Number of classes for classification head
+        in_channels: Number of input channels
+        input_size: Input image size
+        patch_layer: Layer used to transform image to patches. Possible values are
+            `patch_embeddings` and `hybrid_embeddings`.
+        patch_nb_blocks: When `patch_layer="hybrid_embeddings`, this is the number of
+            residual blocks in each stage. Set to `()` to use only the stem.
+        patch_size: Patch size; Image size must be multiple of patch size. For hybrid
+            embedding layer, this patch size is applied after the convolutional layers.
         embed_dim: Embedding dimension
-        depth: Depth of transformer (number of encoder blocks)
+        nb_blocks: Depth of transformer (number of encoder blocks)
         nb_heads: Number of self-attention heads
-        mlp_ratio: ratio of mlp hidden dim to embedding dim
-        qkv_bias: enable bias for qkv if True
-        representation_size: enable and set representation layer (pre-logits) to this
+        mlp_ratio: Ratio of mlp hidden dim to embedding dim
+        qkv_bias: Enable bias for qkv if True
+        representation_size: Enable and set representation layer (pre-logits) to this
             value if set
-        distilled: model includes a distillation token and head as in DeiT models
-        drop_rate: dropout rate
-        attn_drop_rate: attention dropout rate
-        norm_layer: normalization layer
-        act_layer: activation function
+        distilled: Model includes a distillation token and head as in DeiT models
+        drop_rate: Dropout rate
+        attn_drop_rate: Attention dropout rate
+        drop_path_rate: Dropout rate for stochastic depth
+        norm_layer: Normalization layer
+        act_layer: Activation function
     """
 
-    def __post_init__(self):
-        self.transform_weights["pos_embed"] = ViT.transform_pos_embed
-
     @property
     def nb_tokens(self) -> int:
+        """Number of special tokens"""
         return 2 if self.distilled else 1
 
     @property
     def grid_size(self) -> Tuple[int, int]:
-        return (
+        grid_size = (
             self.input_size[0] // self.patch_size,
             self.input_size[1] // self.patch_size,
         )
+        if self.patch_layer == "hybrid_embeddings":
+            # 2 reductions in the stem, 1 reduction in each stage except the first one
+            reductions = 2 + max(len(self.patch_nb_blocks) - 1, 0)
+            stride = 2 ** reductions
+            grid_size = (grid_size[0] // stride, grid_size[1] // stride)
+        return grid_size
 
     @property
     def nb_patches(self) -> int:
+        """Number of patches without class and distillation tokens."""
         return self.grid_size[0] * self.grid_size[1]
 
+    @property
+    def transform_weights(self):
+        return {"pos_embed": ViT.transform_pos_embed}
 
-class MultiHeadAttention(tf.keras.layers.Layer):
+
+class ViTMultiHeadAttention(tf.keras.layers.Layer):
     def __init__(self, cfg: ViTConfig, **kwargs):
         super().__init__(**kwargs)
         head_dim = cfg.embed_dim // cfg.nb_heads
@@ -133,7 +160,8 @@ class Block(tf.keras.layers.Layer):
         self.norm_layer = norm_layer_factory(cfg.norm_layer)
 
         self.norm1 = self.norm_layer(name="norm1")
-        self.attn = MultiHeadAttention(cfg, name="attn")
+        self.attn = ViTMultiHeadAttention(cfg, name="attn")
+        self.drop_path = DropPath(drop_prob=cfg.drop_path_rate)
         self.norm2 = self.norm_layer(name="norm2")
         self.mlp = MLP(
             hidden_dim=int(cfg.embed_dim * cfg.mlp_ratio),
@@ -144,11 +172,80 @@ class Block(tf.keras.layers.Layer):
         )
 
     def call(self, x, training=False):
-        x = x + self.attn(self.norm1(x, training=training), training=training)
-        x = x + self.mlp(self.norm2(x, training=training), training=training)
+        shortcut = x
+        x = self.norm1(x, training=training)
+        x = self.attn(x, training=training)
+        x = self.drop_path(x, training=training)
+        x = x + shortcut
+
+        shortcut = x
+        x = self.norm2(x, training=training)
+        x = self.mlp(x, training=training)
+        x = self.drop_path(x, training=training)
+        x = x + shortcut
         return x
 
 
+class HybridEmbeddings(tf.keras.layers.Layer):
+    """
+    CNN feature map embedding
+
+    Extract feature map from CNN, flatten, project to embedding dim.
+    """
+
+    def __init__(
+        self,
+        in_channels: int,
+        input_size: tuple,
+        nb_blocks: tuple,
+        patch_size: int,
+        embed_dim: int,
+        drop_path_rate: float,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        if nb_blocks == ():
+            self.backbone = ResNetV2Stem(
+                stem_type="same",
+                stem_width=64,
+                conv_padding="same",
+                preact=False,
+                act_layer="relu",
+                norm_layer="group_norm",
+                name="backbone",
+            )
+        else:
+            backbone_cfg = ResNetV2Config(
+                nb_classes=0,
+                in_channels=in_channels,
+                input_size=input_size,
+                nb_blocks=nb_blocks,
+                preact=False,
+                stem_type="same",
+                global_pool="",
+                conv_padding="same",
+                drop_path_rate=drop_path_rate,
+            )
+            self.backbone = ResNetV2(backbone_cfg, name="backbone")
+
+        self.projection = tf.keras.layers.Conv2D(
+            filters=embed_dim,
+            kernel_size=patch_size,
+            strides=patch_size,
+            use_bias=True,
+            name="proj",
+        )
+
+    def call(self, x, training=False, return_shape=False):
+        x = self.backbone(x, training=training)
+        x = self.projection(x)
+
+        # Change the 2D spatial dimensions to a single temporal dimension.
+        batch_size, height, width = tf.unstack(tf.shape(x)[:3])
+        x = tf.reshape(tensor=x, shape=(batch_size, height * width, -1))
+        return (x, (height, width)) if return_shape else x
+
+
 @keras_serializable
 class ViT(tf.keras.Model):
     cfg_class = ViTConfig
@@ -159,39 +256,31 @@ class ViT(tf.keras.Model):
         self.norm_layer = norm_layer_factory(cfg.norm_layer)
         self.cfg = cfg
 
-        self.patch_embed = PatchEmbeddings(
-            patch_size=cfg.patch_size,
-            embed_dim=cfg.embed_dim,
-            norm_layer="",  # ViT does not use normalization in patch embeddings
-            name="patch_embed",
-        )
-        self.cls_token = self.add_weight(
-            shape=(1, 1, cfg.embed_dim),
-            initializer="zeros",
-            trainable=True,
-            name="cls_token",
-        )
-        self.dist_token = (
-            self.add_weight(
-                shape=(1, 1, cfg.embed_dim),
-                initializer="zeros",
-                trainable=True,
-                name="dist_token",
+        if cfg.patch_layer == "patch_embeddings":
+            self.patch_embed = PatchEmbeddings(
+                patch_size=cfg.patch_size,
+                embed_dim=cfg.embed_dim,
+                norm_layer="",  # ViT does not use normalization in patch embeddings
+                name="patch_embed",
             )
-            if cfg.distilled
-            else None
-        )
-        self.pos_embed = self.add_weight(
-            shape=(1, cfg.nb_patches + cfg.nb_tokens, cfg.embed_dim),
-            initializer="zeros",
-            trainable=True,
-            name="pos_embed",
-        )
+        elif cfg.patch_layer == "hybrid_embeddings":
+            self.patch_embed = HybridEmbeddings(
+                in_channels=cfg.in_channels,
+                input_size=cfg.input_size,
+                nb_blocks=cfg.patch_nb_blocks,
+                patch_size=cfg.patch_size,
+                embed_dim=cfg.embed_dim,
+                drop_path_rate=cfg.drop_path_rate,
+                name="patch_embed",
+            )
+        else:
+            raise ValueError(f"Unknown patch layer: {cfg.patch_layer}.")
+        self.cls_token = None
+        self.dist_token = None
+        self.pos_embed = None
         self.pos_drop = tf.keras.layers.Dropout(rate=cfg.drop_rate)
 
-        # Note: We did not implement stochastic depth, since none of the pretrained
-        # timm models use it
-        self.blocks = [Block(cfg, name=f"blocks/{j}") for j in range(cfg.depth)]
+        self.blocks = [Block(cfg, name=f"blocks/{j}") for j in range(cfg.nb_blocks)]
         self.norm = self.norm_layer(name="norm")
 
         # Some models have a representation layer on top of cls token
@@ -222,54 +311,55 @@ class ViT(tf.keras.Model):
         else:
             self.head_dist = None
 
+    def build(self, input_shape):
+        self.cls_token = self.add_weight(
+            shape=(1, 1, self.cfg.embed_dim),
+            initializer="zeros",
+            trainable=True,
+            name="cls_token",
+        )
+        self.dist_token = (
+            self.add_weight(
+                shape=(1, 1, self.cfg.embed_dim),
+                initializer="zeros",
+                trainable=True,
+                name="dist_token",
+            )
+            if self.cfg.distilled
+            else None
+        )
+        self.pos_embed = self.add_weight(
+            shape=(1, self.cfg.nb_patches + self.cfg.nb_tokens, self.cfg.embed_dim),
+            initializer="zeros",
+            trainable=True,
+            name="pos_embed",
+        )
+
     @property
     def dummy_inputs(self) -> tf.Tensor:
-        return tf.zeros((1, *self.cfg.input_size, self.cfg.in_chans))
+        return tf.zeros((1, *self.cfg.input_size, self.cfg.in_channels))
+
+    @property
+    def feature_names(self) -> List[str]:
+        return (
+            ["patch_embedding"]
+            + [f"block_{j}" for j in range(self.cfg.nb_blocks)]
+            + ["features_all", "features", "logits"]
+        )
 
     def transform_pos_embed(self, target_cfg: ViTConfig):
-        return self.interpolate_pos_embed(target_cfg.input_size)
-
-    def interpolate_pos_embed(self, input_size: Tuple[int, int]):
-        """
-        This method allows to interpolate the pre-trained position encodings, to be
-        able to use the model on higher resolution images.
-
-        Args:
-            input_size: Input size to which position embeddings should be adapted
-
-        Returns:
-            Position embeddings (including class tokens) appropriate to input_size
-        """
-        cfg = self.cfg
-
-        if input_size == cfg.input_size:
-            return self.pos_embed  # No interpolation needed
-
-        src_pos_embed = self.pos_embed[:, cfg.nb_tokens :]
-        src_pos_embed = tf.reshape(
-            src_pos_embed, shape=(1, *cfg.grid_size, cfg.embed_dim)
+        return interpolate_pos_embeddings(
+            pos_embed=self.pos_embed,
+            src_grid_size=self.cfg.grid_size,
+            tgt_grid_size=target_cfg.grid_size,
+            nb_tokens=self.cfg.nb_tokens,
         )
-        tgt_grid_size = (
-            input_size[0] // cfg.patch_size,
-            input_size[1] // cfg.patch_size,
-        )
-        tgt_pos_embed = tf.image.resize(
-            images=src_pos_embed,
-            size=tgt_grid_size,
-            method="bicubic",
-        )
-        tgt_pos_embed = tf.cast(tgt_pos_embed, dtype=src_pos_embed.dtype)
-        tgt_pos_embed = tf.reshape(tgt_pos_embed, shape=(1, -1, cfg.embed_dim))
-        tgt_pos_embed = tf.concat(
-            (self.pos_embed[:, : cfg.nb_tokens], tgt_pos_embed), axis=1
-        )
-        return tgt_pos_embed
 
-    def forward_features(self, x, training=False):
+    def forward_features(self, x, training=False, return_features=False):
+        features = {}
         batch_size = tf.shape(x)[0]
-        input_size = tf.shape(x)[1:3]
 
-        x = self.patch_embed(x)
+        x, grid_size = self.patch_embed(x, return_shape=True)
         cls_token = tf.repeat(self.cls_token, repeats=batch_size, axis=0)
         if not self.cfg.distilled:
             x = tf.concat((cls_token, x), axis=1)
@@ -279,31 +369,46 @@ class ViT(tf.keras.Model):
         if not self.cfg.interpolate_input:
             x = x + self.pos_embed
         else:
-            x = x + self.interpolate_pos_embed(input_size)
+            pos_embed = interpolate_pos_embeddings(
+                self.pos_embed,
+                src_grid_size=self.cfg.grid_size,
+                tgt_grid_size=grid_size,
+                nb_tokens=self.cfg.nb_tokens,
+            )
+            x = x + pos_embed
         x = self.pos_drop(x, training=training)
+        features["patch_embedding"] = x
 
-        for block in self.blocks:
+        for j, block in enumerate(self.blocks):
             x = block(x, training=training)
+            features[f"block_{j}"] = x
         x = self.norm(x, training=training)
+        features["features_all"] = x
 
         if self.cfg.distilled:
             # Here we diverge from timm and return both outputs as one tensor. That way
             # all models always have one output by default
-            return x[:, :2]
+            x = x[:, :2]
         elif self.cfg.representation_size:
-            return self.pre_logits(x[:, 0])
+            x = self.pre_logits(x[:, 0])
         else:
-            return x[:, 0]
+            x = x[:, 0]
+        features["features"] = x
+        return (x, features) if return_features else x
 
-    def call(self, x, training=False):
-        x = self.forward_features(x, training)
+    def call(self, x, training=False, return_features=False):
+        features = {}
+        x = self.forward_features(x, training, return_features)
+        if return_features:
+            x, features = x
         if not self.cfg.distilled:
             x = self.head(x)
         else:
             y = self.head(x[:, 0])
             y_dist = self.head_dist(x[:, 1])
             x = tf.stack((y, y_dist), axis=1)
-        return x
+        features["logits"] = x
+        return (x, features) if return_features else x
 
 
 @register_model
@@ -314,7 +419,7 @@ def vit_tiny_patch16_224():
         url="",
         patch_size=16,
         embed_dim=192,
-        depth=12,
+        nb_blocks=12,
         nb_heads=3,
     )
     return ViT, cfg
@@ -329,7 +434,7 @@ def vit_tiny_patch16_384():
         input_size=(384, 384),
         patch_size=16,
         embed_dim=192,
-        depth=12,
+        nb_blocks=12,
         nb_heads=3,
         crop_pct=1.0,
     )
@@ -344,7 +449,7 @@ def vit_small_patch32_224():
         url="",
         patch_size=32,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
     )
     return ViT, cfg
@@ -359,7 +464,7 @@ def vit_small_patch32_384():
         input_size=(384, 384),
         patch_size=32,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
         crop_pct=1.0,
     )
@@ -374,7 +479,7 @@ def vit_small_patch16_224():
         url="",
         patch_size=16,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
     )
     return ViT, cfg
@@ -389,7 +494,7 @@ def vit_small_patch16_384():
         input_size=(384, 384),
         patch_size=16,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
         crop_pct=1.0,
     )
@@ -408,7 +513,7 @@ def vit_base_patch32_224():
         url="",
         patch_size=32,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -427,7 +532,7 @@ def vit_base_patch32_384():
         input_size=(384, 384),
         patch_size=32,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         crop_pct=1.0,
     )
@@ -446,7 +551,7 @@ def vit_base_patch16_224():
         url="",
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -465,7 +570,7 @@ def vit_base_patch16_384():
         input_size=(384, 384),
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         crop_pct=1.0,
     )
@@ -484,7 +589,7 @@ def vit_base_patch8_224():
         url="",
         patch_size=8,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -501,7 +606,7 @@ def vit_large_patch32_224():
         url="",
         patch_size=32,
         embed_dim=1024,
-        depth=24,
+        nb_blocks=24,
         nb_heads=16,
     )
     return ViT, cfg
@@ -520,7 +625,7 @@ def vit_large_patch32_384():
         input_size=(384, 384),
         patch_size=32,
         embed_dim=1024,
-        depth=24,
+        nb_blocks=24,
         nb_heads=16,
         crop_pct=1.0,
     )
@@ -539,7 +644,7 @@ def vit_large_patch16_224():
         url="",
         patch_size=16,
         embed_dim=1024,
-        depth=24,
+        nb_blocks=24,
         nb_heads=16,
     )
     return ViT, cfg
@@ -558,7 +663,7 @@ def vit_large_patch16_384():
         input_size=(384, 384),
         patch_size=16,
         embed_dim=1024,
-        depth=24,
+        nb_blocks=24,
         nb_heads=16,
         crop_pct=1.0,
     )
@@ -576,7 +681,7 @@ def vit_base_patch32_sam_224():
         url="",
         patch_size=32,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -593,7 +698,7 @@ def vit_base_patch16_sam_224():
         url="",
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -612,7 +717,7 @@ def vit_tiny_patch16_224_in21k():
         nb_classes=21843,
         patch_size=16,
         embed_dim=192,
-        depth=12,
+        nb_blocks=12,
         nb_heads=3,
     )
     return ViT, cfg
@@ -631,7 +736,7 @@ def vit_small_patch32_224_in21k():
         nb_classes=21843,
         patch_size=32,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
     )
     return ViT, cfg
@@ -650,7 +755,7 @@ def vit_small_patch16_224_in21k():
         nb_classes=21843,
         patch_size=16,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
     )
     return ViT, cfg
@@ -670,7 +775,7 @@ def vit_base_patch32_224_in21k():
         nb_classes=21843,
         patch_size=32,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -690,7 +795,7 @@ def vit_base_patch16_224_in21k():
         nb_classes=21843,
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -709,7 +814,7 @@ def vit_base_patch8_224_in21k():
         nb_classes=21843,
         patch_size=8,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
     )
     return ViT, cfg
@@ -730,7 +835,7 @@ def vit_large_patch32_224_in21k():
         nb_classes=21843,
         patch_size=32,
         embed_dim=1024,
-        depth=24,
+        nb_blocks=24,
         nb_heads=16,
         representation_size=1024,
     )
@@ -751,7 +856,7 @@ def vit_large_patch16_224_in21k():
         nb_classes=21843,
         patch_size=16,
         embed_dim=1024,
-        depth=24,
+        nb_blocks=24,
         nb_heads=16,
     )
     return ViT, cfg
@@ -772,7 +877,7 @@ def vit_huge_patch14_224_in21k():
         nb_classes=21843,
         patch_size=14,
         embed_dim=1280,
-        depth=32,
+        nb_blocks=32,
         nb_heads=16,
         representation_size=1280,
     )
@@ -790,7 +895,7 @@ def deit_tiny_patch16_224():
         url="",
         patch_size=16,
         embed_dim=192,
-        depth=12,
+        nb_blocks=12,
         nb_heads=3,
         mean=IMAGENET_DEFAULT_MEAN,
         std=IMAGENET_DEFAULT_STD,
@@ -809,7 +914,7 @@ def deit_small_patch16_224():
         url="",
         patch_size=16,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
         mean=IMAGENET_DEFAULT_MEAN,
         std=IMAGENET_DEFAULT_STD,
@@ -828,7 +933,7 @@ def deit_base_patch16_224():
         url="",
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         mean=IMAGENET_DEFAULT_MEAN,
         std=IMAGENET_DEFAULT_STD,
@@ -848,7 +953,7 @@ def deit_base_patch16_384():
         input_size=(384, 384),
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         crop_pct=1.0,
         mean=IMAGENET_DEFAULT_MEAN,
@@ -868,7 +973,7 @@ def deit_tiny_distilled_patch16_224():
         url="",
         patch_size=16,
         embed_dim=192,
-        depth=12,
+        nb_blocks=12,
         nb_heads=3,
         distilled=True,
         mean=IMAGENET_DEFAULT_MEAN,
@@ -889,7 +994,7 @@ def deit_small_distilled_patch16_224():
         url="",
         patch_size=16,
         embed_dim=384,
-        depth=12,
+        nb_blocks=12,
         nb_heads=6,
         distilled=True,
         mean=IMAGENET_DEFAULT_MEAN,
@@ -910,7 +1015,7 @@ def deit_base_distilled_patch16_224():
         url="",
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         distilled=True,
         mean=IMAGENET_DEFAULT_MEAN,
@@ -932,7 +1037,7 @@ def deit_base_distilled_patch16_384():
         input_size=(384, 384),
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         distilled=True,
         crop_pct=1.0,
@@ -956,7 +1061,7 @@ def vit_base_patch16_224_miil_in21k():
         input_size=(224, 224),
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         qkv_bias=False,
         crop_pct=0.875,
@@ -980,7 +1085,7 @@ def vit_base_patch16_224_miil():
         input_size=(224, 224),
         patch_size=16,
         embed_dim=768,
-        depth=12,
+        nb_blocks=12,
         nb_heads=12,
         qkv_bias=False,
         crop_pct=0.875,
