@@ -21,6 +21,7 @@ from enum import Enum
 
 import numpy
 import tensorflow as tf
+from tensorflow.python.keras import backend as K
 
 logger = logging.getLogger(__name__)
 
@@ -35,9 +36,7 @@ class TransposeType(Enum):
     CONV2D = "conv2d"
 
 
-def convert_tf_weight_name_to_pt_weight_name(
-    tf_name, start_prefix_to_remove="", tf_weight_shape=None
-):
+def convert_tf_weight_name_to_pt_weight_name(tf_name, tf_weight_shape=None):
     """
     Convert a TF 2.0 model variable name in a pytorch model weight name.
 
@@ -101,62 +100,37 @@ def convert_tf_weight_name_to_pt_weight_name(
     if tf_name[-1] == "moving_variance":
         tf_name[-1] = "running_var"
 
-    # Remove prefix if needed
+    # Put name together
     tf_name = ".".join(tf_name)
-    if start_prefix_to_remove:
-        tf_name = tf_name.replace(start_prefix_to_remove, "", 1)
 
     return tf_name, transpose
 
 
-#####################
-# PyTorch => TF 2.0 #
-#####################
-
-
 def load_pytorch_weights_in_tf2_model(
     tf_model, pt_state_dict, tf_inputs=None, allow_missing_keys=False
 ):
     """Load pytorch state_dict in a TF 2.0 model."""
-    try:
-        import tensorflow as tf  # noqa: F401
-        import torch  # noqa: F401
-        from tensorflow.python.keras import backend as K
-    except ImportError:
-        logger.error(
-            "Loading a PyTorch model in TensorFlow, requires both PyTorch and TensorFlow to be installed. Please see "
-            "https://pytorch.org/ and https://www.tensorflow.org/install/ for installation instructions."
-        )
-        raise
-
     if tf_inputs is None:
         tf_inputs = tf_model.dummy_inputs
 
     if tf_inputs is not None:
         tf_model(tf_inputs, training=False)  # Make sure model is built
-    # Adapt state dict - TODO remove this and update the AWS weights files instead
-    # Convert old format to new format if needed from a PyTorch state_dict
+
+    # Adapt pt state dict. TF "beta" -> PT "bias"
+    # But some models have PT weight "beta" (ResMLP affine layer)
+    # To fix that we need to change PT name to "bias" first...
     old_keys = []
     new_keys = []
     for key in pt_state_dict.keys():
         new_key = None
-        if "gamma" in key:
-            new_key = key.replace("gamma", "weight")
-        if "beta" in key:
-            new_key = key.replace("beta", "bias")
+        if key.endswith(".beta"):
+            new_key = key.replace(".beta", ".bias")
         if new_key:
             old_keys.append(key)
             new_keys.append(new_key)
     for old_key, new_key in zip(old_keys, new_keys):
         pt_state_dict[new_key] = pt_state_dict.pop(old_key)
 
-    # Make sure we are able to load PyTorch base models as well as derived models (with heads)
-    # TF models always have a prefix, some of PyTorch models (base ones) don't
-    start_prefix_to_remove = ""
-    # TODO: Do we need this?
-    # if not any(s.startswith(tf_model.base_model_prefix) for s in pt_state_dict.keys()):
-    #     start_prefix_to_remove = tf_model.base_model_prefix + "."
-
     symbolic_weights = tf_model.trainable_weights + tf_model.non_trainable_weights
     tf_loaded_numel = 0
     weight_value_tuples = []
@@ -166,7 +140,6 @@ def load_pytorch_weights_in_tf2_model(
         sw_name = symbolic_weight.name
         name, transpose = convert_tf_weight_name_to_pt_weight_name(
             sw_name,
-            start_prefix_to_remove=start_prefix_to_remove,
             tf_weight_shape=symbolic_weight.shape,
         )
 
@@ -175,11 +148,11 @@ def load_pytorch_weights_in_tf2_model(
             if allow_missing_keys:
                 missing_keys.append(name)
                 continue
-            # TODO: Do we need this?
-            # elif tf_model._keys_to_ignore_on_load_missing is not None:
-            #     # authorized missing keys don't have to be loaded
-            #     if any(re.search(pat, name) is not None for pat in tf_model._keys_to_ignore_on_load_missing):
-            #         continue
+            keys_to_ignore = getattr(tf_model, "keys_to_ignore_on_load_missing", None)
+            if keys_to_ignore is not None:
+                # authorized missing keys don't have to be loaded
+                if any(re.search(pat, sw_name) is not None for pat in keys_to_ignore):
+                    continue
             print(name, sw_name)
             raise AttributeError(f"{name} not found in PyTorch model")
 
@@ -212,7 +185,6 @@ def load_pytorch_weights_in_tf2_model(
             raise e
 
         tf_loaded_numel += array.size
-        # logger.warning(f"Initialize TF weight {symbolic_weight.name}")
 
         weight_value_tuples.append((symbolic_weight, array))
         all_pytorch_weights.discard(name)
@@ -225,43 +197,34 @@ def load_pytorch_weights_in_tf2_model(
     logger.info(f"Loaded {tf_loaded_numel:,} parameters in the TF 2.0 model.")
 
     unexpected_keys = list(all_pytorch_weights)
-
-    # TODO: Do we need this?
-    # if tf_model._keys_to_ignore_on_load_missing is not None:
-    #     for pat in tf_model._keys_to_ignore_on_load_missing:
-    #         missing_keys = [k for k in missing_keys if re.search(pat, k) is None]
-    # if tf_model._keys_to_ignore_on_load_unexpected is not None:
-    #     for pat in tf_model._keys_to_ignore_on_load_unexpected:
-    #         unexpected_keys = [k for k in unexpected_keys if re.search(pat, k) is None]
+    # PyTorch BN layers track number of batches, but TF does not, so these weights
+    # will always be left over.
+    unexpected_keys = [
+        key for key in unexpected_keys if "num_batches_tracked" not in key
+    ]
 
     if len(unexpected_keys) > 0:
         logger.warning(
-            f"Some weights of the PyTorch model were not used when "
-            f"initializing the TF 2.0 model {tf_model.__class__.__name__}: {unexpected_keys}\n"
-            f"- This IS expected if you are initializing {tf_model.__class__.__name__} from a PyTorch model trained on another task "
-            f"or with another architecture (e.g. initializing a TFBertForSequenceClassification model from a BertForPreTraining model).\n"
-            f"- This IS NOT expected if you are initializing {tf_model.__class__.__name__} from a PyTorch model that you expect "
-            f"to be exactly identical (e.g. initializing a TFBertForSequenceClassification model from a BertForSequenceClassification model)."
+            f"Some weights of the PyTorch model were not used when initializing the "
+            f"TF 2.0 model {tf_model.__class__.__name__}: {unexpected_keys}."
         )
     else:
         logger.warning(
-            f"All PyTorch model weights were used when initializing {tf_model.__class__.__name__}.\n"
+            f"All PyTorch model weights were used when initializing "
+            f"{tf_model.__class__.__name__}."
         )
     if len(missing_keys) > 0:
         logger.warning(
-            f"Some weights or buffers of the TF 2.0 model {tf_model.__class__.__name__} were not initialized from the PyTorch model "
-            f"and are newly initialized: {missing_keys}\n"
-            f"You should probably TRAIN this model on a down-stream task to be able to use it for predictions and inference."
+            f"Some weights or buffers of the TF 2.0 model "
+            f"{tf_model.__class__.__name__} were not initialized from the PyTorch "
+            f"model and are newly initialized: {missing_keys}."
         )
     else:
         logger.warning(
-            f"All the weights of {tf_model.__class__.__name__} were initialized from the PyTorch model.\n"
-            f"If your task is similar to the task the model of the checkpoint was trained on, "
-            f"you can already use {tf_model.__class__.__name__} for predictions without further training."
+            f"All the weights of {tf_model.__class__.__name__} were initialized from "
+            f"the PyTorch model.\n"
         )
 
-    return tf_model
-
 
 def load_timm_weights(model: tf.keras.Model, model_name: str):
     """Loads weights from timm in TF model."""
@@ -278,5 +241,16 @@ def load_timm_weights(model: tf.keras.Model, model_name: str):
 
     pt_model = timm.create_model(model_name, pretrained=True)
     pt_state_dict = pt_model.state_dict()
-    model = load_pytorch_weights_in_tf2_model(model, pt_state_dict)
-    return model
+    load_pytorch_weights_in_tf2_model(model, pt_state_dict)
+
+
+def load_pth_url_weights(model: tf.keras.Model, url: str):
+    """Loads weights from a PyTorch .pth file specified by URL."""
+    try:
+        from torch.hub import load_state_dict_from_url
+    except ImportError:
+        logging.error("To load weights from pth weights, torch needs to be installed.")
+        raise
+
+    pt_state_dict = load_state_dict_from_url(url, progress=False, map_location="cpu")
+    load_pytorch_weights_in_tf2_model(model, pt_state_dict)
